function [x] = irls(b,A,p)
% Iteratively-reweighted least-square (IRLS) method
% solves: min lp-norm(x) s.t. Ax=b.
%
% Inputs:
%  b: Mx1 vector
%  A: MxN matrix (M<N)
%  p: norm (0<p<=1)
%
% Output
% x: Nx1 vector

x = pinv(A)*b;
e = 1;

while e>1e-13
    w = power(x.*x + e,p/2-1);
    Q = diag(1./w);
    x = Q*A'*inv(A*Q*A')*b;
    e = e/10;
end

